@@ -1,16 +1,14 @@
-# Copyright (c) 2019, Riverbank Computing Limited
+# Copyright (c) 2020, Riverbank Computing Limited
 # All rights reserved.
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
+# This copy of PyQt-builder is licensed for use under the terms of the SIP
+# License Agreement.  See the file LICENSE for more details.
 #
-# 1. Redistributions of source code must retain the above copyright notice,
-#    this list of conditions and the following disclaimer.
+# This copy of PyQt-builder may also used under the terms of the GNU General
+# Public License v2 or v3 as published by the Free Software Foundation which
+# can be found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this
+# package.
 #
-# 2. Redistributions in binary form must reproduce the above copyright notice,
-#    this list of conditions and the following disclaimer in the documentation
-#    and/or other materials provided with the distribution.
-# 
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -87,6 +85,71 @@ class QmakeBuilder(Builder):
                     # This will exist (and we can't check anyway).
                     self.spec = 'macx-clang'
 
+            # The remaining options influenced by the Qt configuration are
+            # project options.
+            project = self.project
+
+            # Determine the target platform, ignoring any current value.
+            xspec = self.qt_configuration['QMAKE_XSPEC']
+
+            # Note that the order of these tests is important.
+            if 'android' in xspec:
+                py_platform = 'android'
+            elif 'ios' in xspec:
+                py_platform = 'ios'
+            elif 'macx' in xspec:
+                py_platform = 'darwin'
+            elif 'wasm' in xspec:
+                py_platform = 'wasm'
+            elif 'win32' in xspec:
+                py_platform = 'win32'
+            else:
+                # Treat everything else as Linux.
+                py_platform = 'linux'
+
+            project.py_platform = py_platform
+
+            # Set the default minimum GLIBC version.
+            if not project.minimum_glibc_version:
+                if self.qt_version >= 0x060000:
+                    project.minimum_glibc_version = '2.28'
+                else:
+                    project.minimum_glibc_version = '2.17'
+
+            # Set the default minimum macOS version.
+            if not project.minimum_macos_version:
+                if self.qt_version >= 0x060000:
+                    project.minimum_macos_version = '10.14'
+                elif self.qt_version >= 0x050e00:
+                    project.minimum_macos_version = '10.13'
+                elif self.qt_version >= 0x050c00:
+                    project.minimum_macos_version = '10.12'
+                elif self.qt_version >= 0x050a00:
+                    project.minimum_macos_version = '10.11'
+                elif self.qt_version >= 0x050900:
+                    project.minimum_macos_version = '10.10'
+                else:
+                    project.minimum_macos_version = '10.9'
+
+            # Set the default name of the sip module.
+            if not project.sip_module:
+                project.sip_module = 'PyQt{}.sip'.format(self.qt_version >> 16)
+
+            # Set the default ABI version of the sip module.
+            if not project.abi_version:
+                # The defaults should really be the .0 versions but we
+                # originally didn't specify a minor version so that SIP
+                # defaulted to the latest version it knew about (which may vary
+                # according to the SIP version).  We therefore specify the
+                # minimum version needed by the current versions of PyQt at the
+                # time of writing.  If later versions of PyQt have different
+                # requirements then they will explicitly specify the required
+                # version.
+                if project.sip_module == 'PyQt5.sip':
+                    project.abi_version = '12.8'
+                elif project.sip_module == 'PyQt6.sip':
+                    project.abi_version = '13.1'
+
         super().apply_user_defaults(tool)
 
     def build_executable(self, buildable, *, fatal=True):
@@ -207,6 +270,10 @@ class QmakeBuilder(Builder):
         options = super().get_options()
 
         # Add our new options.
+        options.append(
+                Option('jobs', option_type=int,
+                        help="run N make jobs in parallel", metavar='N'))
+
         options.append(
                 Option('make', option_type=bool, inverted=True,
                         help="do not run make or nmake", tools=['build']))
@@ -295,7 +362,12 @@ class QmakeBuilder(Builder):
 
         pro_lines = ['TEMPLATE = lib']
 
-        pro_lines.append('CONFIG += warn_on exceptions_off')
+        pro_lines.append('CONFIG += warn_on')
+
+        if buildable.exceptions:
+            pro_lines.append('CONFIG += exceptions')
+        else:
+            pro_lines.append('CONFIG += exceptions_off')
 
         if buildable.static:
             pro_lines.append('CONFIG += staticlib hide_symbols')
@@ -551,12 +623,17 @@ macx {
         directory.
         """
 
+        project = self.project
+
         args = [self._find_make()]
 
         if install:
             args.append('install')
+        elif project.py_platform != 'win32' and self.jobs:
+            args.append('-j')
+            args.append(str(self.jobs))
 
-        self.project.run_command(args)
+        project.run_command(args)
 
     def _run_qmake(self, pro_name, fatal=True, recursive=False):
         """ Run qmake against a .pro file.  fatal is set if a qmake failure is
@@ -642,6 +719,21 @@ macx {
         sources = [self.qmake_quote(f) for f in buildable.sources]
         pro_lines.append('SOURCES = {}'.format(' '.join(sources)))
 
+        # Add any extras.
+        if buildable.extra_compile_args:
+            pro_lines.append(
+                    'QMAKE_CXXFLAGS += {}'.format(
+                            ' '.join(buildable.extra_compile_args)))
+
+        if buildable.extra_link_args:
+            pro_lines.append(
+                    'QMAKE_LFLAGS += {}'.format(
+                            ' '.join(buildable.extra_link_args)))
+
+        if buildable.extra_objects:
+            objects = [self.qmake_quote(f) for f in buildable.objects]
+            pro_lines.append('OBJECTS += {}'.format(' '.join(objects)))
+
     def _write_pro_file(self, pro_fn, pro_lines):
         """ Write a .pro file. """
 
